import unittest
import os
import logging
from MQTTSimpleClient.my_logger import Logger
from MQTTSimpleClient.configuration import Configuration


class TestLogger(unittest.TestCase):
    def setUp(self):
        # Creare una configurazione di test per evitare di sporcare la configurazione reale
        conf_content = '''<config>
            <log>
                <folder>
                    <path type="string">./test_logs</path>
                    <name type="string">test.log</name>
                </folder>
                <formatter type="string">%(asctime)s - %(name)s - %(levelname)s - %(message)s</formatter>
                <RotatingFileHandler>
                    <file_size type="int">1048576</file_size>
                    <backup_count type="int">1</backup_count>
                </RotatingFileHandler>
            </log>
        </config>'''

        os.makedirs('./test_logs', exist_ok=True)
        conf_path = './test_logs/test_conf.xml'
        with open(conf_path, 'w') as f:
            f.write(conf_content)

        # Inizializzare la configurazione con il file di test
        self.config = Configuration(custom_conf_path=conf_path)
        self.logger = Logger('test_script')

    def tearDown(self):
        # Pulire i file di log creati durante i test
        if os.path.exists('./test_logs/test.log'):
            os.remove('./test_logs/test.log')
        if os.path.exists('./test_logs/test_conf.xml'):
            os.remove('./test_logs/test_conf.xml')
        if os.path.exists('./test_logs'):
            os.rmdir('./test_logs')

    def test_logger_levels(self):
        logger = self.logger.get_logger()
        logger.debug('This is a debug message')
        logger.info('This is an info message')
        logger.warning('This is a warning message')
        logger.error('This is an error message')
        logger.critical('This is a critical message')

        logger.handlers[0].flush()  # Ensure all log messages are flushed to the file

        with open('./test_logs/test.log', 'r') as f:
            log_content = f.read()
            self.assertIn('This is a debug message', log_content)
            self.assertIn('This is an info message', log_content)
            self.assertIn('This is a warning message', log_content)
            self.assertIn('This is an error message', log_content)
            self.assertIn('This is a critical message', log_content)

    def test_logger_script_name(self):
        logger = self.logger.get_logger()
        logger.info('Test script name')

        logger.handlers[0].flush()  # Ensure all log messages are flushed to the file

        with open('./test_logs/test.log', 'r') as f:
            log_content = f.read()
            self.assertIn('test_script', log_content)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
"""
Created on Fri Oct 27 14:51:40 2023

@author: sPK-Server
"""

import paho.mqtt.client as mqtt
from time import sleep
from .my_logger import Logger

# Create a logger
logger = Logger(__name__.split(".")[-1]).get_logger()

import threading
from typing import Optional


class MqttClient:
    _instances = {}
    _lock = threading.Lock()

    def __new__(cls, broker_host: str, broker_port: int, client_id: str,
                username: Optional[str] = None, password: Optional[str] = None, **kwargs):
        with cls._lock:
            if client_id not in cls._instances:
                instance = super().__new__(cls)
                cls._instances[client_id] = instance
            return cls._instances[client_id]

    def __init__(self, broker_host: str, broker_port: int, client_id: str,
                 username: Optional[str] = None, password: Optional[str] = None, **kwargs):
        if not hasattr(self, 'initialized'):
            self.__broker_host = broker_host
            self.__broker_port = broker_port
            self.__client_id = client_id
            self.__username = username
            self.__password = password
            self.__client = mqtt.Client(client_id=client_id)
            self.__connected: bool = False
            self.__topic = ""
            self.__qos = 0

            # Connect MQTT event handlers
            self.__client.on_connect = self.__on_connect
            self.__client.on_publish = self.__on_publish
            self.__client.on_disconnect = self.__on_disconnect

            self.initialized = True
            logger.info(f"Module initialized")

    def __on_connect(self, client, userdata, flags, rc) -> None:
        if rc == 0:
            self.__connected = True
            logger.info(f"Client <{self.__client_id}> - Connected to the MQTT broker {self.__broker_host}")
        else:
            logger.error(f"Client <{self.__client_id}> - MQTT broker connection failed due to: " + str(rc))

    def __on_disconnect(self, client, userdata, rc):
        logger.info(f"Client <{self.__client_id}> - Disconnected from MQTT broker {self.__broker_host} cod. {rc}")

    def __on_publish(self, client, userdata, mid) -> None:
        logger.info(f"Client <{self.__client_id}> - Message successfully published Mid: <{str(mid)}> topic <{self.__topic}> qos <{self.__qos}>")

    def connect(self, asyncronous=False) -> None:
        logger.info(f"Client <{self.__client_id}> - Test Connection MQTT broker <{self.__broker_host}>")
        if self.__username and self.__password:
            self.__client.username_pw_set(self.__username, self.__password)
        if asyncronous:
            self.__client.connect_async(self.__broker_host, self.__broker_port, 60)
        else:
            self.__client.connect(self.__broker_host, self.__broker_port, 60)
        sleep(0.1)

    def publish(self, topic: str, message: str, qos=0, retain=False) -> None:
        self.__topic = topic
        self.__qos = qos
        try:
            self.__client.publish(topic, message, qos, retain)
        except Exception as e:
            logger.error(f"Client <{self.__client_id}> - not connected to the MQTT broker <{self.__broker_host}> -> "
                         f"Unable to publish message: {e}")
        sleep(0.1)

    def run(self):
        self.__client.loop_start()
        logger.info(f"Client <{self.__client_id}> - Client Loop Start")

    def run_forever(self):
        logger.info(f"Client <{self.__client_id}> - Client Loop_Forever Start")
        self.__client.loop_forever()

    def disconnect(self) -> None:
        self.__client.disconnect()
        self.__client.loop_stop()
        self.__connected = False

    @property
    def client(self):
        return self.__client

    @property
    def connected(self) -> bool:
        return self.__connected

    @property
    def broker_host(self):
        return self.__broker_host

    @property
    def client_id(self):
        return self.__client_id


class MqttSubscriber:
    __instances = {}

    def __init__(self, client: MqttClient, topic: str, qos: int = 0, **callback):

        self.__client = client
        self.__topic = topic
        self.__qos = qos

        # Connect MQTT event handlers
        self.__client.client.on_connect = self.__on_connect

        # Load subscription callback function
        self.__callback = {}
        for key, value in callback.items():
            if callable(value):
                self.__callback[key] = value
            else:
                logger.warning(f"Il valore passato per '{key}' non è una funzione. Verrà ignorato.")

        # self.__subscribe()
        MqttSubscriber.__instances[topic] = self

    def __on_connect(self, client, userdata, flags, rc) -> None:
        if rc == 0:
            self.__connected = True
            MqttSubscriber.__subscribe_all()
            logger.info(f"Client <{self.__client.client_id}> - connected to the MQTT broker {self.__client.broker_host}")
        else:
            logger.error(f"Client <{self.__client.client_id}> - MQTT broker connection failed due to: " + str(rc))

    def __on_message(self, client, userdata, message):
        logger.info(f"Client <{self.__client.client_id}> - message received on topic {message.topic}:"
                    f" {message.payload.decode()[:40]} ... ")
        self.__message = message

        for k, func in self.__callback.items():
            sleep(0.1)
            try:
                func(self.__message)
                sleep(0.1)
            except TypeError as e:
                logger.error(f"Client <{self.__client.client_id}> - Error in __on_message() callback: {str(e)}")

    def __subscribe(self) -> None:

        logger.info(f"Client <{self.__client.client_id}> - Test subscription: topic <{self.__topic}>")

        self.__client.client.subscribe(self.__topic, self.__qos)
        self.__client.client.message_callback_add(self.__topic, self.__on_message)
        logger.info(f"Client <{self.__client.client_id}> - Subscription established on topic '{self.__topic}':"
                    f" QoS <{str(self.__qos)}>, callback <{list(self.__callback.keys())}>")
        sleep(0.1)

    def unsubscribe(self) -> None:
        if self.__topic:

            self.__client.client.unsubscribe(self.__topic)
            logger.info(f"Client <{self.__client.client_id}> - Subscription cancelled to topic " + self.__topic)
            self.__topic = ''
        else:
            logger.error("No active subscription")

    @classmethod
    def __subscribe_all(cls):
        for instance in cls.__instances.values():
            instance.__subscribe()

    @classmethod
    def unsubscribe_all(cls):
        for instance in cls.__instances.values():
            instance.unsubscribe()


if __name__ == "__main__":

    def func1(message, casa):
        print(f"Funzione 1: {message.payload.decode()}")


    def func2(message):
        print(f"Funzione 2: {message.payload.decode()}")


    def func3(message):
        print(f"Funzione 3: {message.payload.decode()}")


    mqtt_client = MqttClient("10.7.68.9", 1883, f"{__name__}_test")
    mqtt_client.connect()
    mqtt_client.run()
    if mqtt_client.connected:
        # Pubblica un messaggio
        mqtt_client.publish("Postman",
                            '{"InfluxDB": {"databases": "manz_dbs, manz_calculated_db", "start": '
                            '"2023-06-01T00:00:00Z", "stop": "2023-07-01T00:00:00Z"}}')

    # Test di connessione
    if mqtt_client.connected:
        # Sottoscrivi a un topic
        sub1 = MqttSubscriber(mqtt_client, "Test")

        sub2 = MqttSubscriber(mqtt_client, "Postman", funzione1=func1)

        sub3 = MqttSubscriber(mqtt_client, "Funzione",
                              funzione2=func2, funzione3=func3, funzione4=4)

    sleep(0.1)
    input("Premi Enter per terminare la sottoscrizione...")
    sub1.unsubscribe()
    sub2.unsubscribe()
    sub3.unsubscribe()

    sleep(0.1)
    input("Premi Enter per disconnettersi...")
    mqtt_client.disconnect()



# -*- coding: utf-8 -*-

"""
  Authors: Riccardo Filomena
  Company: Manz IT
  Address: Via S. Lorenzo, 19, 40037 Sasso Marconi (BO)
  Mail: rfilomena@manz.com
  Date: 2024-01-22
"""

"""
  Project: DumpData
  
  configuration.py
  
  Created by Riccardo Filomena
  Property of MANZ ITALY S.R.L.
  Via S. Lorenzo, 19, 40037 Sasso Marconi (BO).
"""

import os
import sys
from lxml import etree as ET

sys.path.append(os.path.dirname(os.path.abspath(__file__)))


def read_xml(root, temp=None):
    """ Reading a file '.xml'.

        input:
            ----------------------------
            - root: Python tree
                subtree's root

            - temp: Python dictionary containing a part of the tree
                default value: dict()

        __output:
            ----------------------------
            - temp: Python dictionary containing all the tree read from the file '.xml'
    """

    if temp is None:
        temp = {}
    try:
        children = root.getchildren()

        if children != list():
            for i, child in enumerate(children):
                temp[child.tag] = dict()
                temp[child.tag] = read_xml(children[i], temp[child.tag])
        else:
            if root.attrib['type'] == 'int':
                return int(root.text)
            elif root.attrib['type'] == 'float':
                return float(root.text)
            elif root.attrib['type'] == 'string':
                return str(root.text)
            elif root.attrib['type'] == 'bool':
                return root.text == 'True'
            else:
                return str(root.text)

        return temp
    except Exception as e:
        print('Problem detected [read_xml]: ', str(e))
        return dict()


class Configuration:
    def __init__(self, custom_conf_path=None):
        self.__debug = False
        self.__parameters = dict()
        self.__set_parameters(custom_conf_path)

    def __set_parameters(self, custom_conf_path) -> None:
        conf_folder = 'cfg'
        conf_name = 'conf.xml'
        default_conf_path = os.path.abspath(os.path.join(os.path.dirname(__file__), conf_folder + os.sep + conf_name))
        conf_path = custom_conf_path if custom_conf_path and os.path.isfile(custom_conf_path) else default_conf_path

        tree = ET.parse(conf_path)
        root = tree.getroot()
        dictionary = dict()

        self.__parameters = read_xml(root, dictionary)
        self.__printer_debug(self.__debug, 'Configuration read successfully!')

    @property
    def parameters(self) -> dict:
        return self.__parameters

    def __printer_debug(self, debug, *var):
        if debug:
            print(*var)


# Instantiation of the class "Configuration" to make it global.
cfg = Configuration()



import os
import sys
import logging
from logging.handlers import RotatingFileHandler
from .configuration import cfg

sys.path.append(os.path.dirname(os.path.abspath(__file__)))


def create_folder(path: str) -> None:
    if not os.path.exists(path):
        os.makedirs(path)


class Logger:
    __instance = None

    def __new__(cls, *args, **kwargs):
        if cls.__instance is None:
            cls.__instance = super().__new__(cls)
        return cls.__instance

    def __init__(self, script_name):
        self.__path = cfg.parameters['log']['folder']['path']
        self.__name = cfg.parameters['log']['folder']['name']
        self.__filename = self.__path + os.sep + self.__name
        self.__formatter = cfg.parameters['log']['formatter']
        self.__log_size = cfg.parameters['log']['RotatingFileHandler']['file_size']
        self.__backup_count = cfg.parameters['log']['RotatingFileHandler']['backup_count']

        # Create folder is not exist
        create_folder(self.__path)

        # Create a _logger
        self.__logger = logging.getLogger(script_name)

        # Load Configuration
        self.__configure()

    def __configure(self):
        # set Level
        self.__logger.setLevel(logging.INFO)

        # Create a file handler
        file_handler = RotatingFileHandler(self.__filename,
                                           maxBytes=self.__log_size,
                                           backupCount=self.__backup_count)
        file_handler.setLevel(logging.INFO)

        # Create a stream handler
        stream_handler = logging.StreamHandler()
        stream_handler.setLevel(logging.INFO)

        # Create a formatter
        formatter = logging.Formatter(self.__formatter)

        # Add the formatter to the handlers
        file_handler.setFormatter(formatter)
        stream_handler.setFormatter(formatter)

        # Add the handlers to the _logger
        self.__logger.addHandler(file_handler)
        self.__logger.addHandler(stream_handler)

    def get_logger(self) -> object:
        return self.__logger



import unittest
import os
from MQTTSimpleClient.configuration import Configuration


class TestConfiguration(unittest.TestCase):
    def setUp(self):
        # Creare una configurazione di test
        conf_content = '''<config>
            <log>
                <folder>
                    <path type="string">./logs</path>
                    <name type="string">app.log</name>
                </folder>
                <formatter type="string">%(asctime)s - %(name)s - %(levelname)s - %(message)s</formatter>
                <RotatingFileHandler>
                    <file_size type="int">10485760</file_size>
                    <backup_count type="int">5</backup_count>
                </RotatingFileHandler>
            </log>
            <test_values>
                <int_value type="int">123</int_value>
                <float_value type="float">123.456</float_value>
                <string_value type="string">test_string</string_value>
                <bool_value type="bool">True</bool_value>
            </test_values>
        </config>'''
        os.makedirs('./test_cfg', exist_ok=True)
        conf_path = './test_cfg/test_conf.xml'
        with open(conf_path, 'w') as f:
            f.write(conf_content)

        self.config = Configuration(custom_conf_path=conf_path)

    def tearDown(self):
        # Pulire i file di configurazione creati durante i test
        if os.path.exists('./test_cfg/test_conf.xml'):
            os.remove('./test_cfg/test_conf.xml')
        if os.path.exists('./test_cfg'):
            os.rmdir('./test_cfg')

    def test_read_configuration(self):
        parameters = self.config.parameters
        self.assertEqual(parameters['log']['folder']['path'], './logs')
        self.assertEqual(parameters['log']['folder']['name'], 'app.log')
        self.assertEqual(parameters['log']['formatter'], '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        self.assertEqual(parameters['log']['RotatingFileHandler']['file_size'], 10485760)
        self.assertEqual(parameters['log']['RotatingFileHandler']['backup_count'], 5)

    def test_data_formats(self):
        parameters = self.config.parameters
        self.assertEqual(parameters['test_values']['int_value'], 123)
        self.assertEqual(parameters['test_values']['float_value'], 123.456)
        self.assertEqual(parameters['test_values']['string_value'], 'test_string')
        self.assertTrue(parameters['test_values']['bool_value'])


if __name__ == '__main__':
    unittest.main()



from setuptools import setup, find_packages

setup(
    name='MQTTSimpleClient',
    version='0.1.1',
    description='A custom MQTT library for interaction with a broker',
    author='Riccardo Filomena',
    author_email='filomena.riccardo@gmail.com',
    url='https://github.com/filoriccardo/MQTTSimpleClient',
    packages=find_packages(),
    package_data={
        'MQTTSimpleClient': ['cfg/conf.xml'],
    },
    include_package_data=True,
    install_requires=[
        'paho-mqtt',
        'lxml',
        'setuptools'
        # aggiungi altre dipendenze qui
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.10',
)



import unittest
from unittest.mock import MagicMock, patch
from MQTTSimpleClient.MqttPubSub import MqttClient, MqttSubscriber
from time import sleep


class TestMqttClient(unittest.TestCase):
    def setUp(self):
        self.broker_host = '10.7.68.9'
        self.broker_port = 1883
        self.client_id = 'test_client'
        self.mqtt_client = MqttClient(self.broker_host, self.broker_port, self.client_id)

    @patch('MQTTSimpleClient.MqttPubSub.mqtt.Client')
    def test_connect(self, mock_mqtt):
        self.mqtt_client.connect()
        # self.mqtt_client.run()
        sleep(2)
        mock_mqtt().connect.assert_called_with(self.broker_host, self.broker_port, 60)

    # @patch('MQTTSimpleClient.MqttPubSub.mqtt.Client')
    # def test_publish(self, mock_mqtt):
    #     self.mqtt_client.connect()
    #     self.mqtt_client.publish('test/topic', 'test message')
    #     sleep(2)
    #     mock_mqtt().publish.assert_called_with('test/topic', 'test message', 0, False)
    #
    # @patch('MQTTSimpleClient.MqttPubSub.mqtt.Client')
    # def test_subscribe(self, mock_mqtt):
    #     def callback_function(message):
    #         print(f"Message received: {message.payload.decode()}")
    #
    #     self.mqtt_client.connect()
    #     subscriber = MqttSubscriber(self.mqtt_client, 'test/topic', funzione1=callback_function)
    #     self.mqtt_client.client.on_connect(mock_mqtt(), None, None, 0)
    #     sleep(2)
    #     mock_mqtt().subscribe.assert_called_with('test/topic', 0)
    #
    # def test_singleton(self):
    #     client1 = MqttClient(self.broker_host, self.broker_port, self.client_id)
    #     client2 = MqttClient(self.broker_host, self.broker_port, self.client_id)
    #     sleep(2)
    #     self.assertIs(client1, client2)


if __name__ == '__main__':
    unittest.main()
